<?php  
/*  
Ecrit et traite un formulaire sur base d'une description en yaml 
ou d'array associatif
*/
class yamlReadwriter {
    public $content=array();
    public $content_structure;
    public $content_file=false;
    public $destination_folder;
    public $filename;
    public $fileType;

    public $snippet_folder;

    public $messages=array();
    public $compact_output=true;

    public $form_submit="Submit";
    public $form_title="";
    public $form_comment="";
    public $form_delete=false;
    public $containsFiles=false;
    public $form_hidden=array();

    function __construct() {
    }

    public function loadYaml($temp_source){
        if(!is_file($temp_source)){
            $this->messages['error'][]="No file $temp_source !";
            return false;
        }
        $this->content_structure=$temp_source;
        $this->content = Spyc::YAMLLoad($temp_source);
    }

    public function setType($type){
        $this->fileType=$type;
    }

    public function setSnippetFolder($folder){
        if(is_dir($folder)){
            $this->snippet_folder=$folder;
        } else {
            $this->messages['error'][]="No snippet folder! (".$folder.")";
            return false;
        }
    }

    public function setDestinationFolder($folder){
        if(is_dir($folder)){
            $this->destination_folder=$folder;
        } else {
            $this->messages['error'][]="No destination folder! (".$folder.")";
            return false;
        }
    }

    public function loadContentFromYaml($temp_source){
        if(!is_file($temp_source)){
            $this->messages['error'][]="No file $temp_source !";
            return false;
        }

        $array = Spyc::YAMLLoad($temp_source);
        $this->content_file=$temp_source;
        $this->filename=basename($temp_source);
        $this->loadContentFromArray($array);
    }

    public function loadContentFromArray($array){
        // reçoit un array soit simple soit complexe
        foreach($array as $name=>$value){
            if(isset($this->content[$name])){
                // soit c'est un fichier structuré, 
                // soit c'est un multiple

                if(isset($value['value'])){
                    //c'est une valeur structurée
                    $this->content[$name]['value']=$value['value'];
                } else {
                    $this->content[$name]['value']=$value;
                }


                /*
                if(is_array($value) && isset($value['value'])){
                    $this->content[$name]['value']=$value['value'];
                } else {
                    $this->content[$name]['value']=$value;
                }
                */
            }
        }
    }

    public function contentAddField($name,$fields){
        $this->content[$name]=$fields;
    }

    public function contentDeleteField($field){
        if(isset($this->content[$field])){
            unset($this->content[$field]);
        }
    }

    public function writeStructure($dest){

    }

    public function getSimpleArray(){
        $retour=array();
        foreach($this->content as $cle=> $valeur){
            if(is_array($valeur)){
                if($valeur["type"]!="separateur"){
                    $retour[$cle]=$valeur["value"];
                }
            } else {
                $retour[$cle]=$valeur;
            }
        }
        return $retour;
    }

    public function getFilledValues(){
        $values=$this->getSimpleArray();
        foreach($values as $key=>$value){
            if(is_array($value)){
                if(count($value)==0){
                    unset($values[$key]);
                    continue; 
                }
            } else {
                if(empty($value)){
                    unset($values[$key]);
                    continue;
                }
            }
        }
        return $values;
    }

    // renvoyer la liste des fichiers dans ce yaml
    public function getFileList(){
        $retour=array();
        foreach($this->content as $cle=> $valeur){
            if(is_array($valeur) && isset($valeur['type'])){
                if($valeur['type']=="file"){
                    // aplatir les valeur en array
                    if(is_array($valeur["value"])){
                        foreach($valeur["value"] as $file){
                            if($file==""){ continue; }
                            $retour[]=$file; 
                        }
                    } else {
                        if($valeur["value"]==""){ continue; }
                        $retour[]=$valeur["value"]; 
                    }
                } 
            }

        }
        return $retour;
    }




    // ecrire le form
    public function writeForm(){ 
        include($this->snippet_folder."form-start.php");

        // information nécessaies au traitement
        $this->writeHidden("source",$this->filename);
        $this->writeHidden("structure",$this->content_structure);
        $this->writeHidden("destination_folder",$this->destination_folder);
        $this->writeHidden("compact_output",$this->compact_output);
        $this->writeHidden("type",$this->fileType);

        // hidden demandé spécifiquement
        foreach($this->form_hidden as $name => $value){
            $this->writeHidden($name,$value);
        }

        $sourceitem=$this->filename;
        foreach($this->content as $name => $field){
            // si on a un fichier complet (avec type, content etc.)
            // en faire une version 
            $field = (object) $field;

            switch($field->type){

                case "separateur":
                    include($this->snippet_folder."separateur.php");
                    break;

                default:
                    include($this->snippet_folder."field-start.php");
                    include($this->snippet_folder.$field->type.".php");
                    include($this->snippet_folder."field-end.php");
                    break;
            }

        }
        include($this->snippet_folder."form-end.php");

    }

    private function writeHidden($name,$value){
        echo "<input type=\"hidden\" name=\"".$name."\" value=\"".$value."\">";
    }

    public function addHidden($name,$value){
        if($name){
            $this->form_hidden[$name]=$value;
        } 
    }

    public function processPost($savefile=false){
        if(!isset($_POST)){
            $this->messages['error'][]="No post content";
            return false;
        } 
        $fichier=$_POST["source"];
        // charger le fichier d'origine
        unset($_POST["source"]);
        unset($_POST["structure"]);
        unset($_POST["submit"]);

        if(isset($_POST['action'])){
            switch($_POST['action']){
                case("submitform"):
                    unset($_POST['action']);
                    if(isset($_POST['compact_output'])){
                        $this->compact_output=filter_var($_POST['compact_output'], FILTER_VALIDATE_BOOLEAN);
                        unset($_POST['compact_output']);
                    }

                    $this->processText($_POST);
                    $this->processFiles($_FILES);
                    if($savefile){
                        // si le fichier de destination n'existe pas, il sera créé par la fonction writeyaml
                        $this->writeyaml();
                    }
                    $this->messages['success'][]="file updated";
                    break;  
            }
        }
    }

    private function processText($array){
        foreach($array as $cle => $valeur){
            if(isset($this->content[$cle])){
                // ici ajouter un contrôle du type de champ: text, radio, file, rtc
                // attention, les fichiers arrivent en hidden, pas en file

                // si les valeurs sont multiples, vérifier que c'est ok et les traiter
                if(is_array($valeur)){
                    if(!$this->isTagmultiple($cle)){ continue; }
                    $valeurs=array();
                    foreach($valeur as $num=>$val){
                        // filtrer les fichiers temp et les déplacer
                        if($this->content[$cle]['type']=="file"){
                            $val=validateTempfile($val);
                        }

                        $valeurs[]=trim($val);
                    }
                    $this->content[$cle]['value']=$valeurs;
                } else {
                    if($this->content[$cle]['type']=="file"){
                        $valeur=validateTempfile($valeur);
                    }
                    $this->content[$cle]['value']=trim($valeur);

                    $this->messages["success"][] ="entré ".$cle." avec valeur ".trim($valeur);  
                }
            } else {
                // récupération des fichiers à supprimer
                if (strpos($cle, "delete-")===0) {
                    //$ncle=str_replace("delete-","",$this->content[$cle]);
                    if(is_array($valeur)){
                        foreach($valeur as $num=>$filetodelete){
                            include($this->snippet_folder."file-delete.php");
                        }
                    } else {
                        $filetodelete=$valeur;
                        include($this->snippet_folder."file-delete.php"); 
                    }


                    $this->messages["success"][] ="trouvé un doc à supprimer dans ".$cle;  

                } else {
                    $this->messages["error"][] ="clé ".$cle." ignorée";
                }
            }
        }
    }

    public function isTagmultiple($cle){
        if(isset($this->content[$cle]["multiple"]) 
           && $this->content[$cle]["multiple"]=="true"){
            return true;

        } else {
            return false;
        }
    }

    private function processFiles($array){
        if(isset($_FILES) && !empty($_FILES)){
            //echo "Process files";
        }
    }

    public function writeyaml(){
        // contrôle de l'éventuel champ date_creation
        if(isset($this->content['date_creation'])){
            if(!$this->get('date_creation',false)){
                $this->set('date_creation', date('Y-m-d'));
            }
        }

        if($this->compact_output){
            $yaml_str = Spyc::YAMLDump($this->getSimpleArray());
        } else {
            $yaml_str = Spyc::YAMLDump($this->content);
        }
        // si $destination est vide
        // créer le fichier
        if(!$this->filename){
            $this->nameFile();
        }
        if(!$this->destination_folder){
            $this->messages["error"][] ="File ".$this->filename." failed to be written: no destination folder...";
            return false;
        }
        $this->content_file=$this->destination_folder.$this->filename;

        if(file_exists($this->content_file)){
            chmod($this->content_file, 0777); // autoriser la réécriture
        }
        if(file_put_contents($this->content_file,$yaml_str)){
            $this->messages["success"][] ="File ".$this->content_file." written ";
            return $this->filename;
        } else {
            $this->messages["error"][] ="File ".$this->content_file." failed to be written ";
            return false;
        }

    }

    public function deleteyaml(){
        // supprimer les documents attachés et leurs vignettes
        foreach($this->content as $cle=> $valeur){
            if($valeur['type']!="file"){ continue; }
            $co=$valeur["value"];
            if(is_array($co)){
                foreach($co as $dd){
                    $this->deletedoc($dd); 
                }
            } else {
                $this->deletedoc($co);
            }
        }
        // supprimer le fichier
        unlink($this->content_file);
    }

    public function deletedoc($docname){
        @unlink(IMPORTS.$docname);
        $d=pathinfo($docname);
        $dname=$d['filename'];
        $vignettes=glob(CACHE_VIGNETTES.$dname."*");
        foreach($vignettes as $vignette){
            @unlink($vignette); 
        }
    }
    public function cleanContents(){
        foreach($this->content as $cle=>$valeur){
            if(is_array($valeur)){
                if($valeur["type"]!="separateur"){
                    $this->content[$cle]['value']="";
                }
            } else {
                $this->content[$cle]="";
            }
        }
    }

    public function toObjects($arr){
        if(is_array($arr)){
            return json_decode(json_encode($arr));
        }
    }

    public function set($tag, $value){
        if(isset($this->content[$tag])){
            $this->content[$tag]['value']=$value;
        }
    }

    public function get($tag,$default=""){
        $return="";
        if(isset($this->content[$tag])){
            if(is_array($this->content[$tag])){
                $return=$this->content[$tag]["value"];
            } else {
                $return=$this->content[$tag];
            }
        } 
        if($return!=""){
            return $return;
        } else {
            return $default; 
        }
    }

    public function getAsarray($tag){
        if(!$this->get($tag)){
            return array();
        }
        if(is_array($this->get($tag))){
            return $this->get($tag);
        } else {
            return array($this->get($tag));
        }
    }

    public function getParam($tag,$param,$default=""){
        if(isset($this->content[$tag][$param])){
            return $this->content[$tag][$param];
        } else {
            return $default;
        }
    }

    public function getOptions($tag,$separateur=false){
        $info=$this->get($tag);
        // attention, peut être multiple
        if(is_array($info)){
            for($i=0;$i<count($info);$i++){
                if(isset($this->content[$tag]['options'][$info[$i]])){
                    $info[$i]=$this->content[$tag]['options'][$info[$i]];
                }
            }

        } else {
            if(isset($this->content[$tag]['options'][$info])){
                $info= $this->content[$tag]['options'][$info];
            } else {
                $info = $tag;
            }
        }

        if($separateur){
            if(is_array($info)){
                return implode($separateur, $info);
            }
        }
        return $info;
    }

    public function getHtml($tag, $default=""){
        $r=$this->get($tag,$default);
        return nl2br($r);
    }

    public function GetmultipleOrSingle($tag, $default=""){
        if(!$this->isTagmultiple($tag)){ return $this->get($tag, $default); }
        if(is_array($this->get($tag))){ 
            $separator=$this->getParam($tag,"separator",", ");
            return implode($separator, $this->get($tag,""));
        }
        return $this->get($tag);
    }

    public function boolToFrench($t){
        switch($t){
            case "true":
                return "Oui";
                break;
            case "false":
                return "Non";
                break;
            default:
                return $t;
        }
    }

    public function bool($tag){
        if(!$this->get($tag,false)){
            return false; 
        }
        switch($this->get($tag)){
            case "true":
                return true;
                break;
            case "false":
                return false;
                break;
            default:
                return $t;
        }
    }

    // nettoyer avant affichage
    // si c'est un multiple, montrer avec son séparateur
    // si c'est un bollean, afficher oui ou non
    public function getPropre($tag, $default=""){
        $valeurs=$this->get($tag);
        // transformer tout en array
        if(!is_array($valeurs)){
            $valeurs=array($valeurs);
        }
        // passer en revue les valeurs
        for($i=0;$i<count($valeurs);$i++){
            $t=$valeurs[$i];

            if($this->hasOptions($tag)){
                if(isset($this->content[$tag]['options'][$t])){
                    $t=$this->content[$tag]['options'][$t];
                }
            }
            $t=$this->boolToFrench($t);
            $valeurs[$i]=$t;
        }
        $separator=$this->getParam($tag,"separator",", ");
        return implode($separator, $valeurs);
    } 

    public function hasValue($tag){
        if(empty($this->content[$tag]["value"])){
            return false;
        } else {
            return true;
        }
    }

    public function hasOptions($tag){
        if(isset($this->content[$tag]["options"])){
            return true;
        } else {
            return false;
        }
    }

    private function nameFile(){
        $this->filename=date("YmdHis")."_".$this->nettoie_lien($this->get('titre'))."_".uniqid().".yaml";
    }

    private function stripAccents($string){
        $string= \Transliterator::create('NFD; [:Nonspacing Mark:] Remove; NFC')
            ->transliterate($string);
        $string = strtolower(str_replace(" ","_",$string));
        return $string;
    }

    // transforme un lien en un truc uniquement constitué de lettres
    public function nettoie_lien($texte){
        $texte = preg_replace("#[^a-zA-Z]#", "", $texte);
        return $texte;
    }

}